/*
 * This source code (.java) file is Copyright  2000 Artima Software, Inc. All rights reserved. 
 * This file accompanies the Jini Place Service Draft Specification, written by Bill
 * Venners and published on the World Wide Web at:
 * 
 *     http://www.artima.com/jini/cyberspace/DraftSpec.html,
 *
 * This source file may not be copied, modified, or redistributed EXCEPT as allowed
 * by the following statements: From August 11, 2000 through December 31, 2000, you may
 * copy and/or modify these files to test and experiment with the Place API,
 * described in the Jini Place Service Draft Specification. Any bug fixes must be given
 * back to Artima Software, Inc. You may not redistribute this file or any binary (such
 * as .class) files generated from this file. You may not distribute modified versions
 * this files or any binary (such as .class) files generated from modified versions of
 * this file. You may not remove this copyright notice. You may not use this file in
 * printed media without the express permission of Bill Venners. And if that weren't
 * enough, you must destroy all copies of this file, and any binary (such as
 * .class) files generated from this file, by December 31, 2000.
 *
 * ARTIMA SOFTWARE, INC. MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THIS SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
 * BILL VENNERS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY A LICENSEE AS A RESULT
 * OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
*/
package com.artima.cyberspace.ui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class BigIcon extends JPanel {

    private String displayName;
    private String shortDescription;
    private Image icon;

    private static final int xIcon = 26;
    private static final int yIcon = 3;
    private static final int xLabelBox = 4;
    private static final int yLabelBox = 39;
    private static final int widthLabelBox = 76;
    private static final int heightLabelBox = 36;
    
    public BigIcon(String displayName, String shortDescription, Image icon) {

        this.displayName = displayName.trim();
        this.shortDescription = shortDescription;
        this.icon = icon;

    }

    // Make the image 84 width by 82 height. The 32 x 32 icon
    // sits in the center, 3 pixels of border to the north, 4 pixels
    // between it and the selected box of text. The box of text is
    // a maximum of 36 pixels high, with 3 pixels border left at the bottom.
    // To the left and right of the text box are at minimum 4 pixels of
    // border on each side.
    public void paint(Graphics g) {
        
        g.drawImage(icon, 26, 3, this);

        Font labelFont = new Font("Helvetica", Font.PLAIN, 12);
        FontMetrics fm = getFontMetrics(labelFont);
    
        int labelStringWidth = fm.stringWidth(displayName);
    
        String firstLine = displayName;
        String secondLine = "";
        int lineCount = 1;
        if (labelStringWidth > widthLabelBox) {

            // Find maximum number of characters from the
            // beginning of the line that will fit.
            int charsThatFit = findCharsThatFitInOneLine(displayName, fm);
            firstLine = displayName.substring(0, charsThatFit);
            int secondLineStartIndex = charsThatFit;
            lineCount = 2;

            // Search for white space from end
            int spacePos = firstLine.lastIndexOf(' ');
            if (spacePos == -1) {
                // No space. Put in dot dot dots.
                firstLine = firstLine.substring(0, charsThatFit - 3) + "...";

                // Find the next space after the chars consumed on the
                // first line, where the next line will begin.
                int nextSpacePos = displayName.indexOf(' ', charsThatFit);
                if (nextSpacePos != -1) {
                    secondLineStartIndex = nextSpacePos + 1;
                }
                else {
                    lineCount = 1;
                }
            }
            else {
                firstLine = firstLine.substring(0, spacePos);
                charsThatFit = spacePos;
                secondLineStartIndex = spacePos + 1;
            }

            if (lineCount == 2) {

                secondLine = displayName.substring(secondLineStartIndex);
                int secondLineStringWidth = fm.stringWidth(secondLine);

                if (secondLineStringWidth > widthLabelBox) {

                    charsThatFit = findCharsThatFitInOneLine(secondLine, fm);
                    secondLine = secondLine.substring(0, charsThatFit);

                    // Search for white space from end
                    spacePos = secondLine.lastIndexOf(' ');
                    if (spacePos == -1) {

                        // No space. Put in dot dot dots.
                        secondLine = secondLine.substring(0, charsThatFit - 3) + "...";
                    }
                    else if (spacePos < charsThatFit - 3) {
                        secondLine = secondLine.substring(0, spacePos + 1) + "...";
                    }
                    else {
                        secondLine = secondLine.substring(0, charsThatFit - 3) + "...";
                    }
                }
            }
        }

        // Draw first line
        int labelStringAscent = fm.getAscent();
        int labelStringHeight = fm.getHeight();
        int twoLineHeight = fm.getDescent() + labelStringHeight + labelStringAscent;
        int firstLineWidth = fm.stringWidth(firstLine);

        int xStart = xLabelBox + ((widthLabelBox - firstLineWidth) / 2);
        int yStart;

        if (lineCount == 1) {
            yStart = yLabelBox + ((heightLabelBox - labelStringAscent) / 2) + labelStringAscent;
        }
        else {
            yStart = yLabelBox + ((heightLabelBox - twoLineHeight) / 2) + labelStringAscent;
        }
        g.setColor(Color.black);
        g.setFont(labelFont);
        g.drawString(firstLine, xStart, yStart);

        if (lineCount == 2) {

            // Draw second line
            int secondLineWidth = fm.stringWidth(secondLine);

            xStart = xLabelBox + ((widthLabelBox - secondLineWidth) / 2);
            yStart = yLabelBox + ((heightLabelBox - twoLineHeight) / 2) + labelStringAscent + labelStringHeight;

            g.drawString(secondLine, xStart, yStart);
        }
    }

    private int findCharsThatFitInOneLine(String s, FontMetrics fm) {

        int len = s.length();
        int returnVal = len;
        for (int i = 2; i < len; ++i) {
            String sub = s.substring(0, i);
            if (fm.stringWidth(sub) > widthLabelBox) {
                returnVal = i - 1;
                break;
            }
        }

        // Return the number of characters consumed by this line
        return returnVal;
    }

    public Dimension getMaximumSize() {
        return new Dimension(84, 82);
    }

    public Dimension getMinimumSize() {
        return new Dimension(84, 82);
    }

    public Dimension getPreferredSize() {
        return new Dimension(84, 82);
    }
}
